require "colorize"
require "thor"
require "logger"
require "benchmark"

module PackageCloud
  module CLI
    autoload :Distro,      "package_cloud/cli/distro"
    autoload :Entry,       "package_cloud/cli/entry"
    autoload :MasterToken, "package_cloud/cli/master_token"
    autoload :ReadToken,   "package_cloud/cli/read_token"
    autoload :Repository,  "package_cloud/cli/repository"

    class Base < Thor
      class_option "config"
      class_option "url"
      class_option "verbose"

      private
        def config
          $logger = ::Logger.new(STDOUT)
          $verbose = !!options[:verbose]
          if $verbose
            $logger.level = ::Logger::DEBUG
            $logger.debug("verbose mode enabled")
          else
            $logger.level = ::Logger::WARN
          end
          @config ||= begin
            ConfigFile.new(options[:config] || "~/.packagecloud",
                         options[:url] || "https://packagecloud.io").tap(&:read_or_create)
                      end
        end

        def client
          @client ||= Client.new(config)
        end
    end
  end
end
